/* Just a simple example program that creates a terminal in a frame
 * and lets the user interact with it.
 *
 * To compile:
 *    gcc -o boxshell boxshell.c $(pkg-config madtty --cflags --libs)
 */

#include <ncurses.h>

#include <signal.h>
#include <stdio.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <termios.h>
#include <time.h>

#include <madtty/madtty.h>

static unsigned char getout = 0;
static int screen_w, screen_h;
static WINDOW *term_win;

void handler(int signo)
{
    switch (signo) {
      case SIGCHLD:
        getout = 1;
        break;
      case SIGWINCH:
        break;
    }
}

int main(void)
{
    madtty_t *rt;
    int dirty = 0;

    signal(SIGCHLD, handler);
    signal(SIGWINCH, handler);

    madtty_initialize();
    getmaxyx(stdscr, screen_h, screen_w);

    /* create a window with a frame */
    term_win = newwin(screen_h - 2, screen_w - 2, 1, 1);
    rt = madtty_create(screen_h - 2, screen_w - 2);
    {
        const char *path = getenv("SHELL") ?: "/bin/sh";
        const char *args[] = { path, "--login", NULL};

        madtty_forkpty(rt, path, args);
    }

    /* keep reading keypresses from the user and passing them to the terminal;
     * also, redraw the terminal to the window at each iteration */
    while (!getout) {
        fd_set rfds;
        struct timeval tv = { 0, 1000 * 1000 / 50 };
        int ch;

        FD_ZERO(&rfds);
        FD_SET(0, &rfds);
        FD_SET(rt->pty, &rfds);

        if (select(rt->pty + 1, &rfds, NULL, NULL, &tv) > 0) {
            if (FD_ISSET(rt->pty, &rfds)) {
                madtty_process(rt);
                dirty = 1;
            }
        }

        while ((ch = getch()) != ERR) {
#if 0
            if (ch == KEY_F(1)) {
                struct winsize ws = {
                    .ws_row = --rt->rows,
                    .ws_col = --rt->cols,
                };

                erase();
                ioctl(rt->pty, TIOCSWINSZ, &ws);
                wresize(term_win, rt->rows, rt->cols);
            }
#endif
            madtty_keypress(rt, ch); /* pass the keypress for handling */
            dirty = 1;
        }

        if (dirty) {
            madtty_draw(rt, term_win, 0, 0);
            wrefresh(term_win);
            dirty = 0;
        }
    }

    endwin();
    return 0;
}
