/*
    LICENSE INFORMATION:
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License (LGPL) as published by the Free Software Foundation.

    Please refer to the COPYING file for more information.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    Copyright © 2004 Bruno T. C. de Oliveira
    Copyright © 2006 Pierre Habouzit
 */

#ifndef MADTTY_MADTTY_H
#define MADTTY_MADTTY_H

#include <ncurses.h>
#include <stdbool.h>
#include <stdint.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <wchar.h>

#define MAX_CSI_ES_PARAMS 32

typedef struct mtty_row_t mtty_row_t;

typedef struct {
    int   pty;
    pid_t childpid;

    /* flags */
    unsigned seen_input : 1;
    unsigned insert     : 1;
    unsigned escaped    : 1;
    unsigned graphmode  : 1;
    unsigned curshid    : 1;

    /* geometry */
    int rows, cols;

    mtty_row_t *lines;
    mtty_row_t *scroll_top;
    mtty_row_t *scroll_bot;

    /* cursor */
    unsigned curattrs;
    mtty_row_t *curs_row;
    int curs_col;
    int curs_srow, curs_scol;

    /* buffers and parsing state */
    mbstate_t ps;
    char rbuf[BUFSIZ];
    char ebuf[BUFSIZ];
    int  rlen, elen;
} madtty_t;

void madtty_initialize(void);

madtty_t *madtty_create(int rows, int cols);
void madtty_destroy(madtty_t *rt);
pid_t madtty_forkpty(madtty_t *rt, const char *path, const char *argv[]);

int madtty_process(madtty_t *rt);
void madtty_keypress(madtty_t *rt, int keycode);
void madtty_draw(madtty_t *rt, WINDOW *win, int startrow, int startcol);


#endif /* MADTTY_MADTTY_H */
